#include "StdAfx.h"
#include "FtI2C.h"
#include "FtCtrl.h"
#include "FtException.h"

CFtI2C::CFtI2C()
{
	m_nBitSCL_O		= 0;
	m_nBitSDA_O		= 0;
	m_nBitSDA_I		= 0;
	m_nReadStart	= 0;
	m_byPort		= 0x00;
}

CFtI2C::~CFtI2C(void)
{
	Close();
}
 
void CFtI2C::Open(int ftIndex, int baud, int bitSCL_O, int bitSDA_O, int bitSDA_I)
{
	m_nBitSCL_O	= bitSCL_O;
	m_nBitSDA_O	= bitSDA_O;
	m_nBitSDA_I	= bitSDA_I;

	m_nReadStart	= 0;
	m_byPort		= 0x00;
	m_byPort		|= (1<<m_nBitSCL_O);
	m_byPort		|= (1<<m_nBitSDA_O);

	try
	{
		// foCX̃I[v
		m_ft.Open(ftIndex);
		m_ft.SetBaudRate(baud);
		m_ft.SetBitMode(0x00, FT_BITMODE_RESET);
		m_ft.SetBitMode((1<<m_nBitSCL_O) | (1<<m_nBitSDA_O), FT_BITMODE_SYNC_BITBANG);	// BitBang[h
		m_ft.SetLatencyTimer(16);
		m_ft.SetTimeouts(1000, 1000);

		// |[g̏Ԃݒ
		SetSCL(TRUE);
		SetSDA(TRUE);
		Commit();
	}
	catch(CFtException* e)
	{
		e->Delete();
		Close();
	}
}

void CFtI2C::Close()
{
	if(m_ft.IsOpen())
	{
		// Bitbang[h~ă|[gN[Y
		m_ft.SetBitMode(0x00, FT_BITMODE_RESET);
		m_ft.Close();
	}
}

BOOL CFtI2C::IsOpen()
{
	return	m_ft.IsOpen();
}

// SCLo
void CFtI2C::SetSCL(BOOL bHigh)
{
	if(bHigh)
		m_byPort |= (1<<m_nBitSCL_O);
	else
		m_byPort &= ~(1<<m_nBitSCL_O);

	m_abySendBuf.Add(m_byPort);
}

// SDAo
void CFtI2C::SetSDA(BOOL bHigh)
{
	if(bHigh)
		m_byPort |= (1<<m_nBitSDA_O);
	else
		m_byPort &= ~(1<<m_nBitSDA_O);

	m_abySendBuf.Add(m_byPort);
}

// ʐMJn
void CFtI2C::BeginTransmission(BYTE byAddr, BOOL bRead)
{
	BYTE	byData	= (byAddr << 1);

	if(bRead)
		byData |= 0x01;

	// Start Condition
	SetSDA(FALSE);
	SetSCL(FALSE);

	SendByte(byData, TRUE);
}

// ʐMI
void CFtI2C::EndTransmission()
{
	SetSDA(FALSE);
	SetSCL(TRUE);
	SetSDA(TRUE);
}

// foCX֑MEM
void CFtI2C::Commit(CByteArray* pDst)
{
	// M
	DWORD	dwWritten	= 0;
	m_ft.Write(m_abySendBuf.GetData(), m_abySendBuf.GetCount(), &dwWritten);
	m_abySendBuf.RemoveAll();

	// BitBang[hŎ擾Mf[^擾
	CByteArray	abyRecv;
	abyRecv.SetSize(dwWritten);
	DWORD	dwRead	= 0;
	m_ft.Read(abyRecv.GetData(), abyRecv.GetSize(), &dwRead);

	if(pDst != NULL)
	{
		// Mf[^SDA_I𕜌
		int	idx = m_nReadStart;
		int count = abyRecv.GetSize() - m_nReadStart;

		pDst->RemoveAll();

		while(count >= (8*3+3))
		{
			count -= (8*3+3);
			BYTE	byData	= 0x00;

			for(int j = 0; j < 8; j++)
			{
				byData <<= 1;
				idx++;											// 1
				idx++;											// 2
				if((abyRecv[idx++] & (1<<m_nBitSDA_I)) != 0)	// 3
					byData |= 0x01;
			}

			idx++;	// 1:SDA=ACK
			idx++;	// 2:SCL=H
			idx++;	// 3:SCL=L

			pDst->Add(byData);
		}
	}
}

// CӃoCgM
void CFtI2C::ReadBytes(int len)
{
	m_nReadStart	= m_abySendBuf.GetSize();

	while(len-- > 0)
		SendByte(0xFF, (len == 0));
}

// 1oCgM
void CFtI2C::SendByte(BYTE byData, BOOL bAck)
{
	for(int i = 0; i < 8; i++)
	{
		SetSDA((byData & 0x80) != 0);
		byData <<= 1;
		SetSCL(TRUE);
		SetSCL(FALSE);
	}

	// ack
	SetSDA(bAck);
	SetSCL(TRUE);
	SetSCL(FALSE);
}
